@@ -32,18 +32,22 @@ class Clubhouse:
 
     # App/API Information
     API_URL = "https://www.clubhouseapi.com/api"
-    API_BUILD_ID = "304"
-    API_BUILD_VERSION = "0.1.28"
+    API_BUILD_ID = "434"
+    API_BUILD_VERSION = "0.1.40"
     API_UA = f"clubhouse/{API_BUILD_ID} (iPhone; iOS 14.4; Scale/2.00)"
     API_UA_STATIC = f"Clubhouse/{API_BUILD_ID} CFNetwork/1220.1 Darwin/20.3.0"
 
     # Some useful information for commmunication
     PUBNUB_PUB_KEY = "pub-c-6878d382-5ae6-4494-9099-f930f938868b"
     PUBNUB_SUB_KEY = "sub-c-a4abea84-9ca3-11ea-8e71-f2b83ac9263d"
+    PUBNUB_API_URL = "https://clubhouse.pubnubapi.com/v2"
 
     TWITTER_ID = "NyJhARWVYU1X3qJZtC2154xSI"
     TWITTER_SECRET = "ylFImLBFaOE362uwr4jut8S8gXGWh93S1TUKbkfh7jDIPse02o"
 
+    INSTAGRAM_ID = "1352866981588597"
+    INSTAGRAM_CALLBACK = "https://www.joinclubhouse.com/callback/instagram"
+
     AGORA_KEY = "938de3e8055e42b281bb8c6f69c21f78"
     SENTRY_KEY = "5374a416cd2d4009a781b49d1bd9ef44@o325556.ingest.sentry.io/5245095"
     INSTABUG_KEY = "4e53155da9b00728caa5249f2e35d6b3"
@@ -83,10 +87,13 @@ class Clubhouse:
             return func(self, *args, **kwargs)
         return wrap
 
-    def __init__(self, user_id='', user_token='', user_device=''):
-        """ (Clubhouse, str, str, str) -> NoneType
+    def __init__(self, user_id='', user_token='', user_device='', headers=None):
+        """ (Clubhouse, str, str, str, dict) -> NoneType
         Set authenticated information
         """
+        self.HEADERS = dict(self.HEADERS)
+        if isinstance(headers, dict):
+            self.HEADERS.update(headers)
         self.HEADERS['CH-UserID'] = user_id if user_id else "(null)"
         if user_token:
             self.HEADERS['Authorization'] = f"Token {user_token}"
@@ -99,7 +106,7 @@ class Clubhouse:
         >>> str(clubhouse)
         Clubhouse(user_id=(null), user_token=None, user_device=31525f52-6b67-40de-83c0-8f9fe0f6f409)
         """
-        return "Clubhouse(user_Id={}, user_token={}, user_device={}".format(
+        return "Clubhouse(user_Id={}, user_token={}, user_device={})".format(
             self.HEADERS.get('CH-UserID'),
             self.HEADERS.get('Authorization'),
             self.HEADERS.get('CH-DeviceId')
@@ -153,16 +160,19 @@ class Clubhouse:
         req = requests.post(f"{self.API_URL}/resend_phone_number_auth", headers=self.HEADERS, json=data)
         return req.json()
 
-    def complete_phone_number_auth(self, phone_number, verification_code):
-        """ (Clubhouse, str, str) -> dict
+    def complete_phone_number_auth(self, phone_number, rc_token, verification_code):
+        """ (Clubhouse, str, str, str) -> dict
 
         Complete phone number authentication.
+        NOTE: As of June 2021, ReCAPTCHA v3 has been introduced so you need to get that token ready...
         This should return `auth_token`, `access_token`, `refresh_token`, is_waitlisted, ...
         Please note that output may be different depending on the status of the authenticated user
         """
         if self.HEADERS.get("Authorization"):
             raise Exception('Already Authenticatied')
         data = {
+            "device_token": None,
+            "rc_token": rc_token,
             "phone_number": phone_number,
             "verification_code": verification_code
         }
@@ -182,6 +192,16 @@ class Clubhouse:
         req = requests.get(f"{self.API_URL}/check_for_update?{query}", headers=self.HEADERS)
         return req.json()
 
+    @require_authentication
+    def logout(self):
+        """ (Clubhouse) -> dict
+
+        Logout from the app.
+        """
+        data = {}
+        req = requests.post(f"{self.API_URL}/logout", headers=self.HEADERS, json=data)
+        return req.json()
+
     @require_authentication
     def get_release_notes(self):
         """ (Clubhouse) -> dict
@@ -337,13 +357,16 @@ class Clubhouse:
         return req.json()
 
     @require_authentication
-    def get_suggested_follows_similar(self, user_id):
-        """ (Clubhouse, int) -> dict
+    def get_suggested_follows_similar(self, user_id='', username=''):
+        """ (Clubhouse, str, str) -> dict
 
         Get similar users based on the given user.
         """
         data = {
-            "user_id": int(user_id),
+            "user_id": int(user_id) if user_id else None,
+            "username": username if username else None,
+            "query_id": None,
+            "query_result_position": None,
         }
         req = requests.post(f"{self.API_URL}/get_suggested_follows_similar", headers=self.HEADERS, json=data)
         return req.json()
@@ -488,7 +511,10 @@ class Clubhouse:
         """
         data = {
             "club_id": int(club_id),
-            "source_topic_id": source_topic_id
+            "source_topic_id": source_topic_id,
+            "query_id": None,
+            "query_result_position": None,
+            "slug": None,
         }
         req = requests.post(f"{self.API_URL}/get_club", headers=self.HEADERS, json=data)
         return req.json()
@@ -551,6 +577,7 @@ class Clubhouse:
             "channel": channel,
             "attribution_source": attribution_source,
             "attribution_details": attribution_details, # base64_json
+            # logging_context (json of some details)
         }
         req = requests.post(f"{self.API_URL}/join_channel", headers=self.HEADERS, json=data)
         return req.json()
@@ -562,8 +589,7 @@ class Clubhouse:
         Leave the given channel
         """
         data = {
-            "channel": channel,
-            "channel_id": None
+            "channel": channel
         }
         req = requests.post(f"{self.API_URL}/leave_channel", headers=self.HEADERS, json=data)
         return req.json()
@@ -636,13 +662,16 @@ class Clubhouse:
         return req.json()
 
     @require_authentication
-    def get_profile(self, user_id):
-        """ (Clubhouse, str) -> dict
+    def get_profile(self, user_id='', username=''):
+        """ (Clubhouse, str, str) -> dict
 
         Lookup someone else's profile. It is OK to one's own profile with this method.
         """
         data = {
-            "user_id": int(user_id)
+            "query_id": None,
+            "query_result_position": 0,
+            "user_id": int(user_id) if user_id else None,
+            "username": username if username else None
         }
         req = requests.post(f"{self.API_URL}/get_profile", headers=self.HEADERS, json=data)
         return req.json()
@@ -712,6 +741,15 @@ class Clubhouse:
         req = requests.get(f"{self.API_URL}/get_all_topics", headers=self.HEADERS)
         return req.json()
 
+    @require_authentication
+    def get_feed(self):
+        """ (Clubhouse) -> dict
+
+        Get list of channels, current invite status, etc.
+        """
+        req = requests.get(f"{self.API_URL}/get_feed?", headers=self.HEADERS)
+        return req.json()
+
     @require_authentication
     def get_channels(self):
         """ (Clubhouse) -> dict
@@ -1338,7 +1376,6 @@ class Clubhouse:
         req = requests.post(f"{self.API_URL}/cancel_new_channel_invite", headers=self.HEADERS, json=data)
         return req.json()
 
-    @unstable_endpoint
     @require_authentication
     def add_club_admin(self, club_id, user_id):
         """ (Clubhouse, int, int) -> dict
@@ -1352,7 +1389,6 @@ class Clubhouse:
         req = requests.post(f"{self.API_URL}/add_club_admin", headers=self.HEADERS, json=data)
         return req.json()
 
-    @unstable_endpoint
     @require_authentication
     def remove_club_admin(self, club_id, user_id):
         """ (Clubhouse, int, int) -> dict
@@ -1366,7 +1402,6 @@ class Clubhouse:
         req = requests.post(f"{self.API_URL}/remove_club_admin", headers=self.HEADERS, json=data)
         return req.json()
 
-    @unstable_endpoint
     @require_authentication
     def remove_club_member(self, club_id, user_id):
         """ (Clubhouse, int, int) -> dict
@@ -1380,21 +1415,23 @@ class Clubhouse:
         req = requests.post(f"{self.API_URL}/remove_club_member", headers=self.HEADERS, json=data)
         return req.json()
 
-    @unstable_endpoint
     @require_authentication
-    def accept_club_member_invite(self, club_id, source_topic_id=None):
-        """ (Clubhouse, int, int) -> dict
+    def accept_club_member_invite(self, club_id, source_topic_id=None, invite_code=None):
+        """ (Clubhouse, int, int, str) -> dict
 
         Accept Club member invite.
         """
         data = {
             "club_id": int(club_id) if club_id else None,
+            "invite_code": invite_code,
+            "query_id": None,
+            "query_result_position": None,
+            "slug": None,
             "source_topic_id": source_topic_id
         }
         req = requests.post(f"{self.API_URL}/accept_club_member_invite", headers=self.HEADERS, json=data)
         return req.json()
 
-    @unstable_endpoint
     @require_authentication
     def add_club_member(self, club_id, user_id, name, phone_number, message, reason):
         """ (Clubhouse, int, int, str, str, str, unknown) -> dict
@@ -1412,7 +1449,6 @@ class Clubhouse:
         req = requests.post(f"{self.API_URL}/add_club_member", headers=self.HEADERS, json=data)
         return req.json()
 
-    @unstable_endpoint
     @require_authentication
     def get_club_nominations(self, club_id, source_topic_id):
         """ (Club, int, int) -> dict
@@ -1426,7 +1462,6 @@ class Clubhouse:
         req = requests.post(f"{self.API_URL}/get_club_nominations", headers=self.HEADERS, json=data)
         return req.json()
 
-    @unstable_endpoint
     @require_authentication
     def approve_club_nomination(self, club_id, source_topic_id, invite_nomination_id):
         """ (Club, int, int) -> dict
@@ -1441,7 +1476,6 @@ class Clubhouse:
         req = requests.post(f"{self.API_URL}/approve_club_nomination", headers=self.HEADERS, json=data)
         return req.json()
 
-    @unstable_endpoint
     @require_authentication
     def reject_club_nomination(self, club_id, source_topic_id, invite_nomination_id):
         """ (Club, int, int) -> dict
@@ -1456,7 +1490,6 @@ class Clubhouse:
         req = requests.post(f"{self.API_URL}/approve_club_nomination", headers=self.HEADERS, json=data)
         return req.json()
 
-    @unstable_endpoint
     @require_authentication
     def add_club_topic(self, club_id, topic_id):
         """ (Club, int, int) -> dict
@@ -1470,7 +1503,6 @@ class Clubhouse:
         req = requests.post(f"{self.API_URL}/add_club_topic", headers=self.HEADERS, json=data)
         return req.json()
 
-    @unstable_endpoint
     @require_authentication
     def remove_club_topic(self, club_id, topic_id):
         """ (Club, int, int) -> dict
@@ -1484,7 +1516,6 @@ class Clubhouse:
         req = requests.post(f"{self.API_URL}/remove_club_topic", headers=self.HEADERS, json=data)
         return req.json()
 
-    @unstable_endpoint
     @require_authentication
     def get_events_to_start(self):
         """ (Clubhouse) -> dict
@@ -1494,7 +1525,6 @@ class Clubhouse:
         req = requests.get(f"{self.API_URL}/get_events_to_start", headers=self.HEADERS)
         return req.json()
 
-    @unstable_endpoint
     @require_authentication
     def update_is_follow_allowed(self, club_id, is_follow_allowed=True):
         """ (Clubhouse, int, bool) -> dict
@@ -1508,12 +1538,12 @@ class Clubhouse:
         req = requests.post(f"{self.API_URL}/update_is_follow_allowed", headers=self.HEADERS, json=data)
         return req.json()
 
-    @unstable_endpoint
     @require_authentication
-    def update_is_membership_private(self, club_id, is_membership_private):
+    def update_is_membership_private(self, club_id, is_membership_private=False):
         """ (Clubhouse, int, bool) -> dict
 
-        Update membership status of the given Club
+        Update club membership status of the given Club
+        If True, member list will not be shown to public.
         """
         data = {
             "club_id": int(club_id),
@@ -1522,12 +1552,11 @@ class Clubhouse:
         req = requests.post(f"{self.API_URL}/update_is_membership_private", headers=self.HEADERS, json=data)
         return req.json()
 
-    @unstable_endpoint
     @require_authentication
-    def update_is_community(self, club_id, is_community):
+    def update_is_community(self, club_id, is_community=False):
         """ (Clubhouse, int, bool) -> dict
 
-        Update community stat of the given Club
+        Change room start permission. If set False, Admins can only start club rooms.
         """
         data = {
             "club_id": int(club_id),
@@ -1536,7 +1565,6 @@ class Clubhouse:
         req = requests.post(f"{self.API_URL}/update_is_community", headers=self.HEADERS, json=data)
         return req.json()
 
-    @unstable_endpoint
     @require_authentication
     def update_club_description(self, club_id, description):
         """ (Clubhouse, int, str) -> dict
@@ -1550,29 +1578,26 @@ class Clubhouse:
         req = requests.post(f"{self.API_URL}/update_club_description", headers=self.HEADERS, json=data)
         return req.json()
 
-    @unstable_endpoint
     @require_authentication
-    def update_club_rules(self):
-        """ (Clubhouse) -> dict
+    def update_club_rules(self, club_id='', rules=()):
+        """ (Clubhouse, str, list) -> dict
 
-        Not implemented method
+        Update Club's rules (Maximum upto 3 rules)
+        rules: [{'desc': "text", "title": "text"}, ...]
         """
-        raise NotImplementedError("Not Implemented!")
+        data = {
+            "club_id": int(club_id),
+            "rules": rules if rules else [],
+        }
+        req = requests.post(f"{self.API_URL}/update_club_rules", headers=self.HEADERS, json=data)
+        return req.json()
 
-    @unstable_endpoint
     @require_authentication
-    def update_club_topics(self):
-        """ (Clubhouse) -> dict
+    def get_events_for_user(self, user_id='', page_size=25, page=1):
+        """ (Clubhouse, str, int, int) -> dict
 
-        Not implemented method
+        Get events for the specific user.
         """
-        raise NotImplementedError("Not Implemented!")
-
-    @unstable_endpoint
-    @require_authentication
-    def get_events_for_user(self):
-        """ (Clubhouse) -> dict
-
-        Not implemented method
-        """
-        raise NotImplementedError("Not Implemented!")
+        query = f"user_id={user_id}&page_size={page_size}&page={page}"
+        req = requests.get(f"{self.API_URL}/get_events_for_user?{query}", headers=self.HEADERS)
+        return req.json()
